@@ -2,7 +2,7 @@
  mpfr_fits_sint_p, mpfr_fits_slong_p, mpfr_fits_sshort_p,
  mpfr_fits_uint_p, mpfr_fits_ulong_p, mpfr_fits_ushort_p
 
-Copyright 2004-2017 Free Software Foundation, Inc.
+Copyright 2004-2020 Free Software Foundation, Inc.
 Contributed by the AriC and Caramba projects, INRIA.
 
 This file is part of the GNU MPFR Library.
@@ -19,18 +19,10 @@ License for more details.
 
 You should have received a copy of the GNU Lesser General Public License
 along with the GNU MPFR Library; see the file COPYING.LESSER.  If not, see
-http://www.gnu.org/licenses/ or write to the Free Software Foundation, Inc.,
+https://www.gnu.org/licenses/ or write to the Free Software Foundation, Inc.,
 51 Franklin St, Fifth Floor, Boston, MA 02110-1301, USA. */
 
-#ifdef HAVE_CONFIG_H
-# include "config.h"       /* for a build within gmp */
-#endif
-
-#include <stdio.h>
-#include <stdlib.h>
-#include <limits.h>
-
-#include "mpfr-intmax.h"
+#define MPFR_NEED_INTMAX_H
 #include "mpfr-test.h"
 
 #define FTEST_AUX(N,NOT,FCT)                                    \
@@ -47,7 +39,7 @@ http://www.gnu.org/licenses/ or write to the Free Software Foundation, Inc.,
         }                                                       \
       if (__gmpfr_flags != ex_flags)                            \
         {                                                       \
-          unsigned int flags = __gmpfr_flags;                   \
+          mpfr_flags_t flags = __gmpfr_flags;                   \
           printf ("Flags error %d for %s, rnd = %s and x = ",   \
                   N, #FCT,                                      \
                   mpfr_print_rnd_mode ((mpfr_rnd_t) r));        \
@@ -123,7 +115,7 @@ main (void)
 {
   mpfr_exp_t emin, emax;
   mpfr_t x, y;
-  unsigned int flags[2] = { 0, MPFR_FLAGS_ALL }, ex_flags;
+  mpfr_flags_t flags[2] = { 0, MPFR_FLAGS_ALL }, ex_flags;
   int i, r, fi;
 
   tests_start_mpfr ();
@@ -169,9 +161,9 @@ main (void)
 
         /* Check large values (no fit) */
         mpfr_set_ui (x, ULONG_MAX, MPFR_RNDN);
-        mpfr_mul_2exp (x, x, 1, MPFR_RNDN);
+        mpfr_mul_2ui (x, x, 1, MPFR_RNDN);
         CHECK_ALL (8, !!);
-        mpfr_mul_2exp (x, x, 40, MPFR_RNDN);
+        mpfr_mul_2ui (x, x, 40, MPFR_RNDN);
         CHECK_ALL (9, !!);
 
         /* Check a non-integer number just below a power of two. */
@@ -195,7 +187,8 @@ main (void)
             int inv;
 
             mpfr_set_si_2exp (x, -i, -2, MPFR_RNDN);
-            mpfr_rint (y, x, (mpfr_rnd_t) r);
+            /* for RNDF, it fits if it fits when rounding away from zero */
+            mpfr_rint (y, x, r != MPFR_RNDF ? (mpfr_rnd_t) r : MPFR_RNDA);
             inv = MPFR_NOTZERO (y);
             FTEST (80, inv ^ !, mpfr_fits_ulong_p);
             FTEST (81,       !, mpfr_fits_slong_p);
@@ -249,9 +242,9 @@ main (void)
       CHECK_MAX (10, !);
 
       /* Check the limits of the types (except 0 for uintmax_t) */
-      CHECK_LIM (20, MPFR_UINTMAX_MAX, mpfr_set_uj, mpfr_fits_uintmax_p);
-      CHECK_LIM (30, MPFR_INTMAX_MAX, mpfr_set_sj, mpfr_fits_intmax_p);
-      CHECK_LIM (35, MPFR_INTMAX_MIN, mpfr_set_sj, mpfr_fits_intmax_p);
+      CHECK_LIM (20, UINTMAX_MAX, mpfr_set_uj, mpfr_fits_uintmax_p);
+      CHECK_LIM (30, INTMAX_MAX, mpfr_set_sj, mpfr_fits_intmax_p);
+      CHECK_LIM (35, INTMAX_MIN, mpfr_set_sj, mpfr_fits_intmax_p);
 
       /* Check negative op */
       for (i = 1; i <= 4; i++)
@@ -261,7 +254,10 @@ main (void)
           mpfr_set_si_2exp (x, -i, -2, MPFR_RNDN);
           mpfr_rint (y, x, (mpfr_rnd_t) r);
           inv = MPFR_NOTZERO (y);
-          FTEST (80, inv ^ !, mpfr_fits_uintmax_p);
+          if (r != MPFR_RNDF)
+            FTEST (80, inv ^ !, mpfr_fits_uintmax_p);
+          else
+            FTEST (80, !!, mpfr_fits_uintmax_p);
           FTEST (81,       !, mpfr_fits_intmax_p);
         }
     }
